@@ -3,24 +3,26 @@ import importlib
 import inspect
 import json
 import logging
-from multiprocessing import Queue, set_start_method
+from multiprocessing import Process, Queue, set_start_method
 import os
 import pkgutil
 from queue import Empty
 import sys
+from threading import Event as ThreadingEvent
 import time
 import traceback
 from uuid import uuid4
-from typing import Any, Dict, Generator, Iterator, Union, cast
+from typing import Any, Dict, Generator, Iterator, Optional, Union, cast
 
 from flask import Blueprint, Response, current_app as app, request, stream_with_context
 from pydantic import ValidationError
 from vellum_ee.workflows.display.nodes.get_node_display_class import get_node_display_class
 from vellum_ee.workflows.display.types import WorkflowDisplayContext
 
+from vellum.workflows.exceptions import WorkflowInitializationException
 from vellum.workflows.nodes import BaseNode
 from vellum.workflows.utils.names import pascal_to_title_case
-from workflow_server.config import MEMORY_LIMIT_MB
+from workflow_server.config import ENABLE_PROCESS_WRAPPER, MEMORY_LIMIT_MB
 from workflow_server.core.events import (
     SPAN_ID_EVENT,
     STREAM_FINISHED_EVENT,
@@ -30,7 +32,7 @@ from workflow_server.core.events import (
     VembdaExecutionInitiatedBody,
     VembdaExecutionInitiatedEvent,
 )
-from workflow_server.core.executor import stream_node_pebble_timeout, stream_workflow_process_timeout
+from workflow_server.core.executor import stream_node_pebble_timeout, stream_workflow, stream_workflow_process_timeout
 from workflow_server.core.utils import create_vembda_rejected_event, serialize_vembda_rejected_event
 from workflow_server.core.workflow_executor_context import (
     DEFAULT_TIMEOUT_SECONDS,
@@ -73,7 +75,7 @@ def stream_workflow_route() -> Response:
 
     logger.info(
         f"Starting workflow stream, execution ID: {context.execution_id}, "
-        f"process count: {get_active_process_count()}"
+        f"process count: {get_active_process_count()}, process wrapper: {ENABLE_PROCESS_WRAPPER}"
     )
 
     # Create this event up here so timestamps are fully from the start to account for any unknown overhead
@@ -108,27 +110,66 @@ def stream_workflow_route() -> Response:
             headers=headers,
         )
 
-    try:
-        process = stream_workflow_process_timeout(
-            executor_context=context,
-            queue=process_output_queue,
-        )
+    cancel_signal = ThreadingEvent()
+
+    process: Optional[Process] = None
+    if ENABLE_PROCESS_WRAPPER:
+        try:
+            process = stream_workflow_process_timeout(
+                executor_context=context,
+                queue=process_output_queue,
+                cancel_signal=cancel_signal,
+            )
+            increment_process_count(1)
+        except Exception as e:
+            logger.exception(e)
+
+            process_output_queue.put(create_vembda_rejected_event(context, traceback.format_exc()))
+
+        try:
+            first_item = process_output_queue.get(timeout=WORKFLOW_INITIATION_TIMEOUT_SECONDS)
+        except Empty:
+            logger.error("Request timed out trying to initiate the Workflow")
+
+            if process and process.is_alive():
+                process.kill()
+            increment_process_count(-1)
+
+            return Response(
+                json.dumps({"detail": "Request timed out trying to initiate the Workflow"}),
+                status=408,
+                content_type="application/json",
+                headers=headers,
+            )
+    else:
+
+        def workflow_stream_processor() -> Iterator[Union[dict, str]]:
+            span_id_emitted = False
+            try:
+                workflow_iterator, span_id = stream_workflow(
+                    context,
+                    disable_redirect=True,
+                    cancel_signal=cancel_signal,
+                )
+                yield f"{SPAN_ID_EVENT}:{span_id}"
+                span_id_emitted = True
+                for event in workflow_iterator:
+                    yield event
+            except WorkflowInitializationException as e:
+                if not span_id_emitted:
+                    yield f"{SPAN_ID_EVENT}:{uuid4()}"
+
+                yield serialize_vembda_rejected_event(context, str(e))
+            except Exception as e:
+                if not span_id_emitted:
+                    yield f"{SPAN_ID_EVENT}:{uuid4()}"
+
+                logger.exception(e)
+                yield serialize_vembda_rejected_event(context, "Internal Server Error")
+
+        stream_iterator = workflow_stream_processor()
+        first_item = next(stream_iterator)
         increment_process_count(1)
-    except Exception as e:
-        logger.exception(e)
-
-        process_output_queue.put(create_vembda_rejected_event(context, traceback.format_exc()))
-
-    try:
-        first_item = process_output_queue.get(timeout=WORKFLOW_INITIATION_TIMEOUT_SECONDS)
-    except Empty:
-        logger.error("Request timed out trying to initiate the Workflow")
-        return Response(
-            json.dumps({"detail": "Request timed out trying to initiate the Workflow"}),
-            status=408,
-            content_type="application/json",
-            headers=headers,
-        )
 
     if isinstance(first_item, str) and first_item.startswith(SPAN_ID_EVENT):
         span_id = first_item.split(":")[1]
@@ -145,21 +186,30 @@ def stream_workflow_route() -> Response:
     def process_events(queue: Queue) -> Iterator[Union[str, dict]]:
         event: Union[str, dict]
         loops = 0
+        timed_out_time: Optional[float] = None
 
         while True:
             loops += 1
             # Check if we timed out and kill the process if so. Set the timeout a little under what
             # the default is (30m) since the connection limit is 30m and otherwise we may not receive
-            # the timeout event.
-            if min(context.timeout, DEFAULT_TIMEOUT_SECONDS - 90) < (
-                (time.time_ns() - context.request_start_time) / 1_000_000_000
+            # the timeout event. After cancelling the workflow wait 5 seconds for the workflow to emit
+            # any cancel events before ending the stream.
+            if (
+                min(context.timeout, DEFAULT_TIMEOUT_SECONDS - 90)
+                < ((time.time_ns() - context.request_start_time) / 1_000_000_000)
+                and not timed_out_time
             ):
-                logger.error("Workflow timed out")
+                logger.error("Workflow timed out, waiting 5 seconds before ending request...")
+                cancel_signal.set()
+                timed_out_time = time.time()
 
-                if process and process.is_alive():
+            if timed_out_time is not None and timed_out_time + 5 < time.time():
+                logger.warning("Killing request after workflow timeout")
+
+                if ENABLE_PROCESS_WRAPPER and process and process.is_alive():
                     process.kill()
 
-                if process:
+                if not ENABLE_PROCESS_WRAPPER or process:
                     increment_process_count(-1)
 
                 yield VembdaExecutionFulfilledEvent(
@@ -192,15 +242,18 @@ def stream_workflow_route() -> Response:
                 break
 
             try:
-                item = queue.get(timeout=0.1)
-                event = item
+                if ENABLE_PROCESS_WRAPPER:
+                    item = queue.get(timeout=0.1)
+                    event = item
+                else:
+                    event = next(stream_iterator)
             except Empty:
                 # Emit waiting event if were just sitting around to attempt to keep the line
                 # open to trick knative
                 if loops % 20 == 0:
                     yield "WAITING"
 
-                    if process and not process.is_alive():
+                    if ENABLE_PROCESS_WRAPPER and process and not process.is_alive():
                         logger.error("Workflow process exited abnormally")
 
                         yield create_vembda_rejected_event(
@@ -210,6 +263,8 @@ def stream_workflow_route() -> Response:
                         break
 
                 continue
+            except StopIteration:
+                break
             except Exception as e:
                 logger.exception(e)
                 break
@@ -250,14 +305,23 @@ def stream_workflow_route() -> Response:
             # being exceeded.
             app.logger.error("Client disconnected in the middle of the Workflow Stream")
             return
+        except Exception as e:
+            logger.exception("Error during workflow response stream generator", extra={"error": e})
+            yield "\n"
+            yield "END"
+            yield "\n"
+            return
         finally:
-            try:
-                if process and process.is_alive():
-                    process.kill()
-                if process:
-                    increment_process_count(-1)
-            except Exception as e:
-                logger.error("Failed to kill process", e)
+            if ENABLE_PROCESS_WRAPPER:
+                try:
+                    if process and process.is_alive():
+                        process.kill()
+                    if process:
+                        increment_process_count(-1)
+                except Exception as e:
+                    logger.error("Failed to kill process", e)
+            else:
+                increment_process_count(-1)
 
     resp = Response(
         stream_with_context(generator()),
