@@ -84,10 +84,14 @@ def _stream_node_wrapper(executor_context: NodeExecutorContext, queue: Queue) ->
         )
 
 
-def _stream_workflow_wrapper(executor_context: WorkflowExecutorContext, queue: Queue) -> None:
+def _stream_workflow_wrapper(
+    executor_context: WorkflowExecutorContext,
+    queue: Queue,
+    cancel_signal: Optional[ThreadingEvent],
+) -> None:
     span_id_emitted = False
     try:
-        stream_iterator, span_id = stream_workflow(executor_context=executor_context)
+        stream_iterator, span_id = stream_workflow(executor_context=executor_context, cancel_signal=cancel_signal)
 
         queue.put(f"{SPAN_ID_EVENT}:{span_id}")
         span_id_emitted = True
@@ -114,12 +118,14 @@ def _stream_workflow_wrapper(executor_context: WorkflowExecutorContext, queue: Q
 def stream_workflow_process_timeout(
     executor_context: WorkflowExecutorContext,
     queue: Queue,
+    cancel_signal: Optional[ThreadingEvent],
 ) -> Process:
     workflow_process = Process(
         target=_stream_workflow_wrapper,
         args=(
             executor_context,
             queue,
+            cancel_signal,
         ),
     )
     workflow_process.start()
@@ -145,6 +151,7 @@ def stream_workflow_process_timeout(
 def stream_workflow(
     executor_context: WorkflowExecutorContext,
     disable_redirect: bool = True,
+    cancel_signal: Optional[ThreadingEvent] = None,
 ) -> tuple[Iterator[dict], UUID]:
     workflow, namespace = _gather_workflow(executor_context)
     workflow_inputs = _get_workflow_inputs(executor_context)
@@ -164,7 +171,7 @@ def stream_workflow(
     )
 
     cancel_watcher_kill_switch = ThreadingEvent()
-    cancel_signal = ThreadingEvent()
+    cancel_signal = cancel_signal or ThreadingEvent()
     cancel_watcher = CancelWorkflowWatcherThread(
         kill_switch=cancel_watcher_kill_switch,
         execution_id=executor_context.execution_id,
@@ -188,6 +195,7 @@ def stream_workflow(
             cancel_signal=cancel_signal,
             entrypoint_nodes=[run_from_node] if run_from_node else None,
         )
+
     except Exception:
         cancel_watcher_kill_switch.set()
         logger.exception("Failed to generate Workflow Stream")
