@@ -6,10 +6,12 @@ from flask import Flask
 from workflow_server.api.auth_middleware import AuthMiddleware
 from workflow_server.api.healthz_view import bp as healthz_bp
 from workflow_server.api.workflow_view import bp as workflow_bp
-from workflow_server.utils.log_proxy import enable_log_proxy
+from workflow_server.config import is_development
 from workflow_server.utils.sentry import init_sentry
 
-enable_log_proxy()
+# Disabled until we actually use the logs to provide us better debugging
+# enable_log_proxy()
+
 logger = logging.getLogger(__name__)
 logging.basicConfig(
     level=logging.INFO,
@@ -33,6 +35,8 @@ def create_app() -> Flask:
     app.register_blueprint(healthz_bp, url_prefix="/healthz")
     app.register_blueprint(workflow_bp, url_prefix="/workflow")
 
+    logger.info(is_development())
+
     return app
 
 
